@@ -621,7 +621,7 @@ func diffUserTable(
 	}
 
 	if dArgs.diffParts&SchemaOnlyDiff != 0 {
-		err := dw.WriteTableSchemaDiff(ctx, dArgs.toRoot, td)
+		err := dw.WriteTableSchemaDiff(ctx, dArgs.fromRoot, dArgs.toRoot, td)
 		if err != nil {
 			return errhand.VerboseErrorFromError(err)
 		}
@@ -698,8 +698,8 @@ func diffDoltSchemasTable(
 		}
 
 		switch fragmentType {
-		case "view":
-			err := dw.WriteViewDiff(sqlCtx, fragmentName, oldFragment, newFragment)
+		case "event":
+			err := dw.WriteEventDiff(sqlCtx, fragmentName, oldFragment, newFragment)
 			if err != nil {
 				return nil
 			}
@@ -708,6 +708,11 @@ func diffDoltSchemasTable(
 			if err != nil {
 				return nil
 			}
+		case "view":
+			err := dw.WriteViewDiff(sqlCtx, fragmentName, oldFragment, newFragment)
+			if err != nil {
+				return nil
+			}
 		default:
 			cli.PrintErrf("Unrecognized schema element type: %s", fragmentType)
 			continue
