@@ -245,7 +245,7 @@ func (opts *logOpts) parseRefsAndTable(ctx context.Context, apr *argparser.ArgPa
 
 			opts.excludingCommitSpecs = append(opts.excludingCommitSpecs, notCs)
 		} else {
-			argIsRef := actions.ValidateIsRef(ctx, arg, dEnv.DoltDB, dEnv.RepoStateReader())
+			argIsRef := actions.IsValidRef(ctx, arg, dEnv.DoltDB, dEnv.RepoStateReader())
 			// <ref>
 			if argIsRef && !seenRefs[arg] {
 				cs, err := getCommitSpec(arg)
