@@ -53,11 +53,12 @@ import (
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/dsess"
 	"github.com/dolthub/dolt/go/libraries/events"
 	"github.com/dolthub/dolt/go/libraries/utils/filesys"
+	"github.com/dolthub/dolt/go/store/nbs"
 	"github.com/dolthub/dolt/go/store/util/tempfiles"
 )
 
 const (
-	Version = "0.75.9"
+	Version = "0.75.12"
 )
 
 var dumpDocsCommand = &commands.DumpDocsCmd{}
@@ -112,6 +113,7 @@ var doltCommand = cli.NewSubCommandHandler("dolt", "it's git for data", []cli.Co
 	dumpZshCommand,
 	docscmds.Commands,
 	stashcmds.StashCommands,
+	&commands.Assist{},
 })
 
 func init() {
@@ -145,6 +147,10 @@ func main() {
 func runMain() int {
 	args := os.Args[1:]
 
+	if os.Getenv("DOLT_VERBOSE_ASSERT_TABLE_FILES_CLOSED") == "" {
+		nbs.TableIndexGCFinalizerWithStackTrace = false
+	}
+
 	csMetrics := false
 	ignoreLockFile := false
 	if len(args) > 0 {
