@@ -18,29 +18,20 @@ import (
 	"context"
 
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb"
-	"github.com/dolthub/dolt/go/libraries/doltcore/env"
 )
 
-func CheckoutAllTables(ctx context.Context, roots doltdb.Roots, dbData env.DbData) error {
+func CheckoutAllTables(ctx context.Context, roots doltdb.Roots) (doltdb.Roots, error) {
 	tbls, err := doltdb.UnionTableNames(ctx, roots.Working, roots.Staged, roots.Head)
 	if err != nil {
-		return err
+		return doltdb.Roots{}, err
 	}
 
-	return checkoutTables(ctx, dbData, roots, tbls)
+	return MoveTablesFromHeadToWorking(ctx, roots, tbls)
 }
 
 // CheckoutTables takes in a set of tables and docs and checks them out to another branch.
-func CheckoutTables(ctx context.Context, roots doltdb.Roots, dbData env.DbData, tables []string) error {
-	return checkoutTables(ctx, dbData, roots, tables)
-}
-
-func checkoutTables(ctx context.Context, dbData env.DbData, roots doltdb.Roots, tbls []string) error {
-	roots, err := MoveTablesFromHeadToWorking(ctx, roots, tbls)
-	if err != nil {
-		return err
-	}
-	return dbData.Rsw.UpdateWorkingRoot(ctx, roots.Working)
+func CheckoutTables(ctx context.Context, roots doltdb.Roots, tables []string) (doltdb.Roots, error) {
+	return MoveTablesFromHeadToWorking(ctx, roots, tables)
 }
 
 // MoveTablesFromHeadToWorking replaces the tables named from the given head to the given working root, overwriting any
