@@ -22,7 +22,7 @@ import (
 	"fmt"
 	"io"
 
-	"github.com/dolthub/dolt/go/libraries/doltcore/table/editor/creation"
+	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/index"
 
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb"
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb/durable"
@@ -32,7 +32,6 @@ import (
 	"github.com/dolthub/dolt/go/store/prolly"
 	"github.com/dolthub/dolt/go/store/prolly/message"
 	"github.com/dolthub/dolt/go/store/prolly/tree"
-	"github.com/dolthub/dolt/go/store/skip"
 	"github.com/dolthub/dolt/go/store/val"
 )
 
@@ -80,8 +79,9 @@ func mergeTableData(ctx context.Context, tm TableMerger, finalSch schema.Schema,
 	if err != nil {
 		return nil, nil, err
 	}
+
 	// validator shares editor with conflict merge
-	uniq, err := newUniqAddValidator(ctx, finalSch, leftRows, ae, tm)
+	uniq, err := newUniqValidator(ctx, finalSch, tm, ae)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -97,6 +97,12 @@ func mergeTableData(ctx context.Context, tm TableMerger, finalSch schema.Schema,
 			return nil, nil, err
 		}
 
+		cnt, err := uniq.validateDiff(ctx, diff)
+		if err != nil {
+			return nil, nil, err
+		}
+		s.Conflicts += cnt
+
 		switch diff.Op {
 		case tree.DiffOpDivergentModifyConflict, tree.DiffOpDivergentDeleteConflict:
 			s.Conflicts++
@@ -118,11 +124,6 @@ func mergeTableData(ctx context.Context, tm TableMerger, finalSch schema.Schema,
 			if err != nil {
 				return nil, nil, err
 			}
-			cnt, err := uniq.valid(ctx, diff.Op, diff.Key, diff.Right)
-			if err != nil {
-				return nil, nil, err
-			}
-			s.Conflicts += cnt
 		case tree.DiffOpRightModify:
 			s.Modifications++
 			err = pri.merge(ctx, diff)
@@ -133,11 +134,6 @@ func mergeTableData(ctx context.Context, tm TableMerger, finalSch schema.Schema,
 			if err != nil {
 				return nil, nil, err
 			}
-			cnt, err := uniq.valid(ctx, diff.Op, diff.Key, diff.Right)
-			if err != nil {
-				return nil, nil, err
-			}
-			s.Conflicts += cnt
 		case tree.DiffOpRightDelete:
 			s.Deletes++
 			err = pri.merge(ctx, diff)
@@ -158,11 +154,6 @@ func mergeTableData(ctx context.Context, tm TableMerger, finalSch schema.Schema,
 			if err != nil {
 				return nil, nil, err
 			}
-			cnt, err := uniq.valid(ctx, diff.Op, diff.Key, diff.Merged)
-			if err != nil {
-				return nil, nil, err
-			}
-			s.Conflicts += cnt
 		case tree.DiffOpConvergentAdd, tree.DiffOpConvergentModify, tree.DiffOpConvergentDelete:
 			if keyless {
 				s.Conflicts++
@@ -185,12 +176,6 @@ func mergeTableData(ctx context.Context, tm TableMerger, finalSch schema.Schema,
 		return nil, nil, err
 	}
 
-	newUniq, err := uniq.finalize(ctx)
-	if err != nil {
-		return nil, nil, err
-	}
-	s.Conflicts += newUniq
-
 	finalIdxs, err := mergeProllySecondaryIndexes(ctx, tm, leftIdxs, rightIdxs, finalSch, finalRows, conflicts.ae)
 	if err != nil {
 		return nil, nil, err
@@ -241,241 +226,161 @@ func threeWayDiffer(ctx context.Context, tm TableMerger, finalSch schema.Schema)
 	return tree.NewThreeWayDiffer(ctx, leftRows.NodeStore(), leftRows.Tuples(), rightRows.Tuples(), ancRows.Tuples(), vMerger.tryMerge, vMerger.keyless, leftRows.Tuples().Order)
 }
 
-const uniqAddValidatorPendingSize = 650_000
-
-// uniqAddValidator checks whether new additions from the merge-right
-// duplicate secondary index entries.
-type uniqAddValidator struct {
-	name         string
-	rightRootish doltdb.Rootish
-	ae           *prolly.ArtifactsEditor
-	states       []*validateIndexState
-	leftRows     prolly.Map
-	leftSch      schema.Schema
-	primaryKD    val.TupleDesc
-	primaryKB    *val.TupleBuilder
-	batchSize    int
-	pkLen        int
+type uniqValidator struct {
+	src     doltdb.Rootish
+	srcHash hash.Hash
+	edits   *prolly.ArtifactsEditor
+	indexes []uniqIndex
 }
 
-// validateIndexState carries the state required to validate
-// a single unique index.
-type validateIndexState struct {
-	index     schema.Index
-	leftMap   prolly.Map
-	vInfo     []byte
-	pkMapping val.OrdinalMapping
-	secPkMap  val.OrdinalMapping
-	prefixKD  val.TupleDesc
-	prefixKB  *val.TupleBuilder
-	secKb     *val.TupleBuilder
-	batch     *skip.List
-	secCur    *tree.Cursor
-}
+func newUniqValidator(ctx context.Context, sch schema.Schema, tm TableMerger, edits *prolly.ArtifactsEditor) (uniqValidator, error) {
+	srcHash, err := tm.rightSrc.HashOf()
+	if err != nil {
+		return uniqValidator{}, err
+	}
 
-func newUniqAddValidator(ctx context.Context, finalSch schema.Schema, leftRows prolly.Map, ae *prolly.ArtifactsEditor, tm TableMerger) (*uniqAddValidator, error) {
-	indexes := finalSch.Indexes().AllIndexes()
-	primaryKD, _ := leftRows.Descriptors()
-	primaryKB := val.NewTupleBuilder(primaryKD)
+	uv := uniqValidator{
+		src:     tm.rightSrc,
+		srcHash: srcHash,
+		edits:   edits,
+	}
 
-	var states []*validateIndexState
-	for _, index := range indexes {
-		if !index.IsUnique() || !tm.leftSch.Indexes().Contains(index.Name()) {
+	rows, err := tm.leftTbl.GetRowData(ctx)
+	if err != nil {
+		return uniqValidator{}, err
+	}
+	clustered := durable.ProllyMapFromIndex(rows)
+
+	indexes, err := tm.leftTbl.GetIndexSet(ctx)
+	if err != nil {
+		return uniqValidator{}, err
+	}
+
+	for _, def := range sch.Indexes().AllIndexes() {
+		if !def.IsUnique() {
 			continue
+		} else if !tm.leftSch.Indexes().Contains(def.Name()) {
+			continue // todo: how do we validate in this case?
 		}
 
-		is, err := tm.leftTbl.GetIndexSet(ctx)
-		idx, err := is.GetIndex(ctx, tm.leftSch, index.Name())
+		idx, err := indexes.GetIndex(ctx, sch, def.Name())
 		if err != nil {
-			return nil, err
-		}
-		m := durable.ProllyMapFromIndex(idx)
-		if schema.IsKeyless(tm.leftSch) {
-			m = prolly.ConvertToSecondaryKeylessIndex(m)
+			return uniqValidator{}, err
 		}
+		secondary := durable.ProllyMapFromIndex(idx)
 
-		pkMapping := ordinalMappingFromIndex(index)
-		meta, err := makeUniqViolMeta(finalSch, index)
+		u, err := newUniqIndex(sch, def, clustered, secondary)
 		if err != nil {
-			return nil, err
-		}
-		vInfo, err := json.Marshal(meta)
-		if err != nil {
-			return nil, err
+			return uniqValidator{}, err
 		}
+		uv.indexes = append(uv.indexes, u)
+	}
+	return uv, nil
+}
 
-		kd := index.Schema().GetKeyDescriptor()
-		prefixKD := kd.PrefixDesc(index.Count())
-		prefixKB := val.NewTupleBuilder(prefixKD)
-
-		secKb := val.NewTupleBuilder(m.KeyDesc())
-		_, secPkMap := creation.GetIndexKeyMapping(tm.leftSch, index)
-
-		states = append(states, &validateIndexState{
-			index:     index,
-			leftMap:   m,
-			pkMapping: pkMapping,
-			vInfo:     vInfo,
-			prefixKB:  prefixKB,
-			prefixKD:  prefixKD,
-			secKb:     secKb,
-			secPkMap:  secPkMap,
-			batch: skip.NewSkipList(func(left, right []byte) int {
-				return primaryKD.Compare(left, right)
-			}),
+func (uv uniqValidator) validateDiff(ctx context.Context, diff tree.ThreeWayDiff) (conflicts int, err error) {
+	var value val.Tuple
+	switch diff.Op {
+	case tree.DiffOpRightAdd, tree.DiffOpRightModify:
+		value = diff.Right
+	case tree.DiffOpDivergentModifyResolved:
+		value = diff.Merged
+	default:
+		return
+	}
+	for _, idx := range uv.indexes {
+		err = idx.findCollisions(ctx, diff.Key, value, func(k, v val.Tuple) error {
+			conflicts++
+			return uv.insertArtifact(ctx, k, v, idx.meta)
 		})
+		if err != nil {
+			break
+		}
+	}
+	return
+}
+
+func (uv uniqValidator) insertArtifact(ctx context.Context, key, value val.Tuple, meta UniqCVMeta) error {
+	vinfo, err := json.Marshal(meta)
+	if err != nil {
+		return err
+	}
+	cvm := prolly.ConstraintViolationMeta{VInfo: vinfo, Value: value}
+	return uv.edits.ReplaceConstraintViolation(ctx, key, uv.srcHash, prolly.ArtifactTypeUniqueKeyViol, cvm)
+}
+
+type uniqIndex struct {
+	def          schema.Index
+	secondary    prolly.Map
+	clustered    prolly.Map
+	meta         UniqCVMeta
+	prefixDesc   val.TupleDesc
+	secondaryBld index.SecondaryKeyBuilder
+	clusteredBld index.ClusteredKeyBuilder
+}
+
+func newUniqIndex(sch schema.Schema, def schema.Index, clusterd, secondary prolly.Map) (uniqIndex, error) {
+	meta, err := makeUniqViolMeta(sch, def)
+	if err != nil {
+		return uniqIndex{}, err
 	}
 
-	pkLen := tm.leftSch.GetPKCols().Size()
-	if schema.IsKeyless(tm.leftSch) {
-		pkLen = 1
+	if schema.IsKeyless(sch) { // todo(andy): sad panda
+		secondary = prolly.ConvertToSecondaryKeylessIndex(secondary)
 	}
+	p := clusterd.Pool()
 
-	return &uniqAddValidator{
-		name:         tm.name,
-		rightRootish: tm.rightSrc,
-		ae:           ae,
-		states:       states,
-		leftRows:     leftRows,
-		leftSch:      tm.leftSch,
-		primaryKB:    primaryKB,
-		primaryKD:    primaryKD,
-		pkLen:        pkLen,
-		batchSize:    uniqAddValidatorPendingSize,
+	prefixDesc := secondary.KeyDesc().PrefixDesc(def.Count())
+	secondaryBld := index.NewSecondaryKeyBuilder(sch, def, secondary.KeyDesc(), p)
+	clusteredBld := index.NewClusteredKeyBuilder(def, sch, clusterd.KeyDesc(), p)
+
+	return uniqIndex{
+		def:          def,
+		secondary:    secondary,
+		clustered:    clusterd,
+		meta:         meta,
+		prefixDesc:   prefixDesc,
+		secondaryBld: secondaryBld,
+		clusteredBld: clusteredBld,
 	}, nil
 }
 
-// valid queues primary key changes for unique index validation. Primary keys
-// are converted into secondaries for batching ordered lookups.
-func (v *uniqAddValidator) valid(ctx context.Context, op tree.DiffOp, key, value val.Tuple) (int, error) {
-	switch op {
-	case tree.DiffOpDivergentModifyResolved, tree.DiffOpRightAdd, tree.DiffOpRightModify:
-	default:
-		return 0, fmt.Errorf("invalid unique validator diff type: %s", op)
+type collisionFn func(key, value val.Tuple) error
+
+func (idx uniqIndex) findCollisions(ctx context.Context, key, value val.Tuple, cb collisionFn) error {
+	indexKey := idx.secondaryBld.SecondaryKeyFromRow(key, value)
+	if idx.prefixDesc.HasNulls(indexKey) {
+		return nil // NULLs cannot cause unique violations
 	}
-	var conflicts int
-	for i, s := range v.states {
-		secKey, foundNull := v.convertPriToSec(s, key, value)
-		if foundNull {
-			continue
-		}
 
-		s.batch.Put(secKey, value)
-		if s.batch.Count() > v.batchSize {
-			cnt, err := v.flush(ctx, i)
-			conflicts += cnt
-			if err != nil {
-				return 0, err
-			}
-		}
+	var collision val.Tuple
+	err := idx.secondary.GetPrefix(ctx, indexKey, idx.prefixDesc, func(k, _ val.Tuple) (err error) {
+		collision = k
+		return
+	})
+	if err != nil || collision == nil {
+		return err
 	}
-	return conflicts, nil
-}
 
-// convertPriToSec converts a key:value from the primary index into a
-// secondary index key.
-func (v *uniqAddValidator) convertPriToSec(s *validateIndexState, key, value val.Tuple) (val.Tuple, bool) {
-	for to := range s.secPkMap {
-		from := s.secPkMap.MapOrdinal(to)
-		var field []byte
-		if from < v.pkLen {
-			field = key.GetField(from)
-		} else {
-			from -= v.pkLen
-			field = value.GetField(from)
-		}
-		if field == nil {
-			return nil, true
-		}
-		s.secKb.PutRaw(to, field)
+	clusteredKey := idx.clusteredBld.ClusteredKeyFromIndexKey(collision)
+	if bytes.Equal(key, clusteredKey) {
+		return nil // collided with ourselves
 	}
-	return s.secKb.Build(s.leftMap.Pool()), false
-}
 
-// flush performs unique checks on a batch of sorted secondary keys.
-func (v *uniqAddValidator) flush(ctx context.Context, i int) (int, error) {
-	var conflicts int
-	var err error
-
-	s := v.states[i]
-	iter := s.batch.IterAtStart()
-	cur := s.secCur
-	defer s.batch.Truncate()
-	defer func() {
-		s.secCur = nil
-	}()
-
-	var k, value []byte
-	var key val.Tuple
-	for {
-		k, value = iter.Current()
-		key = val.Tuple(k)
-		if key == nil {
-			break
-		}
-		iter.Advance()
-
-		// pluck secondary prefix from secondary key (leading fields)
-		for i := 0; i < s.prefixKD.Count(); i++ {
-			s.prefixKB.PutRaw(i, key.GetField(i))
-		}
-		secKey := s.prefixKB.Build(v.leftRows.Pool())
-
-		if cur == nil {
-			s.secCur, err = tree.NewCursorAtKey(ctx, s.leftMap.NodeStore(), s.leftMap.Node(), val.Tuple(secKey), s.leftMap.KeyDesc())
-			cur = s.secCur
-		}
-
-		err = tree.Seek(ctx, cur, secKey, s.prefixKD)
-		if err != nil {
-			return 0, err
-		}
-		if cur.Valid() {
-			indexK := val.Tuple(cur.CurrentKey())
-			if s.prefixKD.Compare(secKey, indexK) != 0 {
-				continue
-			}
-
-			conflicts++
-
-			// existingPk is the merge-left primary key that
-			// generated the conflicting unique index key
-			existingPK := getPKFromSecondaryKey(v.primaryKB, v.leftRows.Pool(), s.pkMapping, indexK)
-			err = replaceUniqueKeyViolation(ctx, v.ae, v.leftRows, existingPK, v.primaryKD, v.rightRootish, s.vInfo, v.name)
-			if err != nil {
-				return 0, err
-			}
-
-			// newPk is the merge-right primary key whose secondary
-			// index conflicts with existingPk
-			newPK := getPKFromSecondaryKey(v.primaryKB, v.leftRows.Pool(), s.pkMapping, key)
-			err = replaceUniqueKeyViolationWithValue(ctx, v.ae, newPK, value, v.primaryKD, v.rightRootish, s.vInfo, v.name)
-			if err != nil {
-				return 0, err
-			}
+	// |prefix| was non-unique, find the clustered index row that
+	// collided with row(|key|, |value|) and pass both to |cb|
+	err = idx.clustered.Get(ctx, clusteredKey, func(k val.Tuple, v val.Tuple) error {
+		if k == nil {
+			s := idx.clustered.KeyDesc().Format(clusteredKey)
+			return errors.New("failed to find key: " + s)
 		}
+		return cb(k, v)
+	})
+	if err != nil {
+		return err
 	}
-	return conflicts, nil
+	return cb(key, value)
 }
-func (v *uniqAddValidator) finalize(ctx context.Context) (int, error) {
-	var conflicts int
-	for i, _ := range v.states {
-		cnt, err := v.flush(ctx, i)
-		if err != nil {
-			return 0, err
-		}
-		conflicts += cnt
-	}
-	return conflicts, nil
-}
-
-type diffMerger interface {
-	merge(context.Context, tree.ThreeWayDiff) error
-}
-
-var _ diffMerger = (*conflictMerger)(nil)
-var _ diffMerger = (*primaryMerger)(nil)
-var _ diffMerger = (*secondaryMerger)(nil)
 
 // conflictMerger processing primary key diffs
 // with conflict types into artifact table writes.
@@ -556,7 +461,6 @@ type primaryMerger struct {
 	valDesc    val.TupleDesc
 	ns         tree.NodeStore
 	root       tree.Node
-	cur        *tree.Cursor
 	mut        *prolly.MutableMap
 	key, value val.Tuple
 }
