@@ -18,10 +18,8 @@ import (
 	"context"
 
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb/durable"
-
 	"github.com/dolthub/dolt/go/libraries/doltcore/schema"
-	"github.com/dolthub/dolt/go/libraries/doltcore/table/editor/creation"
-	"github.com/dolthub/dolt/go/store/pool"
+	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/index"
 	"github.com/dolthub/dolt/go/store/prolly"
 	"github.com/dolthub/dolt/go/store/val"
 )
@@ -38,7 +36,7 @@ func GetMutableSecondaryIdxs(ctx context.Context, sch schema.Schema, indexes dur
 		if schema.IsKeyless(sch) {
 			m = prolly.ConvertToSecondaryKeylessIndex(m)
 		}
-		mods[i] = NewMutableSecondaryIdx(m, sch, index, m.Pool())
+		mods[i] = NewMutableSecondaryIdx(m, sch, index)
 	}
 	return mods, nil
 }
@@ -55,7 +53,7 @@ func GetMutableSecondaryIdxsWithPending(ctx context.Context, sch schema.Schema,
 		if schema.IsKeyless(sch) {
 			m = prolly.ConvertToSecondaryKeylessIndex(m)
 		}
-		mods[i] = NewMutableSecondaryIdx(m, sch, index, m.Pool())
+		mods[i] = NewMutableSecondaryIdx(m, sch, index)
 		mods[i].mut = mods[i].mut.WithMaxPending(pendingSize)
 	}
 	return mods, nil
@@ -65,32 +63,25 @@ func GetMutableSecondaryIdxsWithPending(ctx context.Context, sch schema.Schema,
 // provides the InsertEntry, UpdateEntry, and DeleteEntry functions which can be
 // used to modify the index based on a modification to corresponding primary row.
 type MutableSecondaryIdx struct {
-	Name     string
-	mut      *prolly.MutableMap
-	keyMap   val.OrdinalMapping
-	pkLen    int
-	keyBld   *val.TupleBuilder
-	syncPool pool.BuffPool
+	Name    string
+	mut     *prolly.MutableMap
+	builder index.SecondaryKeyBuilder
 }
 
 // NewMutableSecondaryIdx returns a MutableSecondaryIdx. |m| is the secondary idx data.
-func NewMutableSecondaryIdx(m prolly.Map, sch schema.Schema, index schema.Index, syncPool pool.BuffPool) MutableSecondaryIdx {
-	kD, _ := m.Descriptors()
-	pkLen, keyMap := creation.GetIndexKeyMapping(sch, index)
+func NewMutableSecondaryIdx(idx prolly.Map, sch schema.Schema, def schema.Index) MutableSecondaryIdx {
+	b := index.NewSecondaryKeyBuilder(sch, def, idx.KeyDesc(), idx.Pool())
 	return MutableSecondaryIdx{
-		Name:     index.Name(),
-		mut:      m.Mutate(),
-		keyMap:   keyMap,
-		pkLen:    pkLen,
-		keyBld:   val.NewTupleBuilder(kD),
-		syncPool: syncPool,
+		Name:    def.Name(),
+		mut:     idx.Mutate(),
+		builder: b,
 	}
 }
 
 // InsertEntry inserts a secondary index entry given the key and new value
 // of the primary row.
 func (m MutableSecondaryIdx) InsertEntry(ctx context.Context, key, newValue val.Tuple) error {
-	newKey := m.mapKeyValue(key, newValue)
+	newKey := m.builder.SecondaryKeyFromRow(key, newValue)
 	err := m.mut.Put(ctx, newKey, val.EmptyTuple)
 	if err != nil {
 		return nil
@@ -101,8 +92,8 @@ func (m MutableSecondaryIdx) InsertEntry(ctx context.Context, key, newValue val.
 // UpdateEntry modifies the corresponding secondary index entry given the key
 // and curr/new values of the primary row.
 func (m MutableSecondaryIdx) UpdateEntry(ctx context.Context, key, currValue, newValue val.Tuple) error {
-	currKey := m.mapKeyValue(key, currValue)
-	newKey := m.mapKeyValue(key, newValue)
+	currKey := m.builder.SecondaryKeyFromRow(key, currValue)
+	newKey := m.builder.SecondaryKeyFromRow(key, newValue)
 
 	err := m.mut.Delete(ctx, currKey)
 	if err != nil {
@@ -118,7 +109,7 @@ func (m MutableSecondaryIdx) UpdateEntry(ctx context.Context, key, currValue, ne
 
 // DeleteEntry deletes a secondary index entry given they key and value of the primary row.
 func (m MutableSecondaryIdx) DeleteEntry(ctx context.Context, key val.Tuple, value val.Tuple) error {
-	currKey := m.mapKeyValue(key, value)
+	currKey := m.builder.SecondaryKeyFromRow(key, value)
 	err := m.mut.Delete(ctx, currKey)
 	if err != nil {
 		return nil
@@ -130,18 +121,3 @@ func (m MutableSecondaryIdx) DeleteEntry(ctx context.Context, key val.Tuple, val
 func (m MutableSecondaryIdx) Map(ctx context.Context) (prolly.Map, error) {
 	return m.mut.Map(ctx)
 }
-
-// mapKeyValue returns the secondary index entry key given the key and value of
-// the corresponding primary row.
-func (m MutableSecondaryIdx) mapKeyValue(k, v val.Tuple) val.Tuple {
-	for to := range m.keyMap {
-		from := m.keyMap.MapOrdinal(to)
-		if from < m.pkLen {
-			m.keyBld.PutRaw(to, k.GetField(from))
-		} else {
-			from -= m.pkLen
-			m.keyBld.PutRaw(to, v.GetField(from))
-		}
-	}
-	return m.keyBld.Build(m.syncPool)
-}
