@@ -83,7 +83,7 @@ func (database) IsReadOnly() bool {
 	return true
 }
 
-func (db database) InitialDBState(ctx context.Context, branch string) (dsess.InitialDbState, error) {
+func (db database) InitialDBState(ctx *sql.Context, branch string) (dsess.InitialDbState, error) {
 	// TODO: almost none of this state is actually used, but is necessary because the current session setup requires a
 	//  repo state writer
 	return dsess.InitialDbState{
@@ -111,6 +111,18 @@ func (db database) EditOptions() editor.Options {
 	return editor.Options{}
 }
 
+func (db database) Revision() string {
+	return ""
+}
+
+func (db database) RevisionType() dsess.RevisionType {
+	return dsess.RevisionTypeNone
+}
+
+func (db database) BaseName() string {
+	return db.Name()
+}
+
 type noopRepoStateWriter struct{}
 
 func (n noopRepoStateWriter) UpdateStagedRoot(ctx context.Context, newRoot *doltdb.RootValue) error {
