@@ -31,6 +31,7 @@ import (
 )
 
 var _ sql.TableFunction = (*LogTableFunction)(nil)
+var _ sql.ExecSourceRel = (*LogTableFunction)(nil)
 
 type LogTableFunction struct {
 	ctx *sql.Context
@@ -201,7 +202,7 @@ func getDoltArgs(ctx *sql.Context, expressions []sql.Expression, name string) ([
 			return args, sql.ErrInvalidArgumentDetails.New(name, expr.String())
 		}
 
-		text, err := types.Text.Convert(childVal)
+		text, _, err := types.Text.Convert(childVal)
 		if err != nil {
 			return nil, err
 		}
