@@ -26,6 +26,7 @@ import (
 	"github.com/dolthub/go-mysql-server/enginetest/scriptgen/setup"
 	"github.com/dolthub/go-mysql-server/sql"
 	"github.com/dolthub/go-mysql-server/sql/mysql_db"
+	"github.com/dolthub/go-mysql-server/sql/rowexec"
 	"github.com/stretchr/testify/require"
 
 	"github.com/dolthub/dolt/go/libraries/doltcore/branch_control"
@@ -80,6 +81,7 @@ var defaultSkippedQueries = []string{
 
 // Setup sets the setup scripts for this DoltHarness's engine
 func (d *DoltHarness) Setup(setupData ...[]setup.SetupScript) {
+	d.closeProvider()
 	d.engine = nil
 	d.provider = nil
 	d.setupData = nil
@@ -177,6 +179,7 @@ func (d *DoltHarness) NewEngine(t *testing.T) (*gms.Engine, error) {
 		if err != nil {
 			return nil, err
 		}
+		e.Analyzer.ExecBuilder = rowexec.DefaultBuilder
 		d.engine = e
 
 		ctx := enginetest.NewContext(d)
@@ -284,6 +287,7 @@ func (d *DoltHarness) SupportsKeylessTables() bool {
 }
 
 func (d *DoltHarness) NewDatabases(names ...string) []sql.Database {
+	d.closeProvider()
 	d.engine = nil
 	d.provider = nil
 
@@ -359,8 +363,25 @@ func (d *DoltHarness) NewDatabaseProvider() sql.MutableDatabaseProvider {
 	return d.provider
 }
 
+func (d *DoltHarness) Close() {
+	d.closeProvider()
+}
+
+func (d *DoltHarness) closeProvider() {
+	if d.provider != nil {
+		dbs := d.provider.AllDatabases(sql.NewEmptyContext())
+		for _, db := range dbs {
+			d.t.Logf("closing %v", db)
+			require.NoError(d.t, db.(sqle.SqlDatabase).DbData().Ddb.Close())
+		}
+	}
+}
+
 func (d *DoltHarness) newProvider() sql.MutableDatabaseProvider {
+	d.closeProvider()
+
 	dEnv := dtestutils.CreateTestEnv()
+	defer dEnv.DoltDB.Close()
 
 	store := dEnv.DoltDB.ValueReadWriter().(*types.ValueStore)
 	store.SetValidateContentAddresses(true)
