@@ -28,7 +28,6 @@ import (
 	"github.com/dolthub/dolt/go/libraries/doltcore/dtestutils"
 	"github.com/dolthub/dolt/go/libraries/doltcore/env"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle"
-	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/json"
 	"github.com/dolthub/dolt/go/store/hash"
 )
 
@@ -141,7 +140,7 @@ func TestDbRevision(t *testing.T) {
 					query: func() string {
 						return fmt.Sprintf("select * from `dolt/%s`.myTable", cm1.String())
 					},
-					rows: []sql.Row{},
+					rows: nil,
 				},
 			},
 		},
@@ -151,6 +150,7 @@ func TestDbRevision(t *testing.T) {
 		t.Run(test.name, func(t *testing.T) {
 			ctx := context.Background()
 			dEnv := dtestutils.CreateTestEnv()
+			defer dEnv.DoltDB.Close()
 
 			setup := append(setupCommon, test.setup...)
 			for _, c := range setup {
@@ -167,14 +167,18 @@ func TestDbRevision(t *testing.T) {
 			require.NotEqual(t, cm3, hash.Hash{})
 
 			for _, a := range test.asserts {
-				makeTestAssertion(t, a, dEnv, root)
+				t.Run(a.query, func(t *testing.T) {
+					makeTestAssertion(t, a, dEnv, root)
+				})
 			}
 			for _, a2 := range test.asserts2 {
 				a := testAssert{
 					query: a2.query(),
 					rows:  a2.rows,
 				}
-				makeTestAssertion(t, a, dEnv, root)
+				t.Run(a.query, func(t *testing.T) {
+					makeTestAssertion(t, a, dEnv, root)
+				})
 			}
 		})
 	}
@@ -194,21 +198,5 @@ func populateCommitHashes(t *testing.T, dEnv *env.DoltEnv, root *doltdb.RootValu
 func makeTestAssertion(t *testing.T, a testAssert, dEnv *env.DoltEnv, root *doltdb.RootValue) {
 	actRows, err := sqle.ExecuteSelect(dEnv, root, a.query)
 	require.NoError(t, err)
-
-	require.Equal(t, len(a.rows), len(actRows))
-	for i := range a.rows {
-		assert.Equal(t, len(a.rows[i]), len(actRows[i]))
-		for j := range a.rows[i] {
-			exp, act := a.rows[i][j], actRows[i][j]
-
-			// special logic for comparing JSONValues
-			if js, ok := exp.(json.NomsJSON); ok {
-				cmp, err := js.Compare(sql.NewEmptyContext(), act.(json.NomsJSON))
-				require.NoError(t, err)
-				assert.Zero(t, cmp)
-			} else {
-				assert.Equal(t, exp, act)
-			}
-		}
-	}
+	assert.Equal(t, a.rows, actRows)
 }
