@@ -96,8 +96,8 @@ func (rrd ReadReplicaDatabase) ValidReplicaState(ctx *sql.Context) bool {
 // InitialDBState implements dsess.SessionDatabase
 // This seems like a pointless override from the embedded Database implementation, but it's necessary to pass the
 // correct pointer type to the session initializer.
-func (rrd ReadReplicaDatabase) InitialDBState(ctx context.Context, branch string) (dsess.InitialDbState, error) {
-	return GetInitialDBState(ctx, rrd, branch)
+func (rrd ReadReplicaDatabase) InitialDBState(ctx *sql.Context, branch string) (dsess.InitialDbState, error) {
+	return initialDBState(ctx, rrd, branch)
 }
 
 func (rrd ReadReplicaDatabase) PullFromRemote(ctx *sql.Context) error {
@@ -379,6 +379,9 @@ func pullBranches(
 
 	_, err := rrd.limiter.Run(ctx, "-all", func() (any, error) {
 		pullErr := rrd.ddb.PullChunks(ctx, rrd.tmpDir, rrd.srcDB, remoteHashes, nil)
+		if pullErr != nil {
+			return nil, pullErr
+		}
 
 	REFS: // every successful pass through the loop below must end with CONTINUE REFS to get out of the retry loop
 		for _, remoteRef := range remoteRefs {
