@@ -166,7 +166,6 @@ func (t DoltTable) LockedToRoot(ctx *sql.Context, root *doltdb.RootValue) (*Dolt
 // Internal interface for declaring the interfaces that read-only dolt tables are expected to implement
 // Add new interfaces supported here, rather than in separate type assertions
 type doltReadOnlyTableInterface interface {
-	sql.Table2
 	sql.TemporaryTable
 	sql.IndexAddressableTable
 	sql.ForeignKeyTable
@@ -457,20 +456,6 @@ func (t *DoltTable) PartitionRows(ctx *sql.Context, partition sql.Partition) (sq
 	return partitionRows(ctx, table, t.sqlSch.Schema, t.projectedCols, partition)
 }
 
-func (t DoltTable) PartitionRows2(ctx *sql.Context, part sql.Partition) (sql.RowIter2, error) {
-	table, err := t.DoltTable(ctx)
-	if err != nil {
-		return nil, err
-	}
-
-	iter, err := partitionRows(ctx, table, t.sqlSch.Schema, t.projectedCols, part)
-	if err != nil {
-		return nil, err
-	}
-
-	return iter.(sql.RowIter2), err
-}
-
 func partitionRows(ctx *sql.Context, t *doltdb.Table, sqlSch sql.Schema, projCols []uint64, partition sql.Partition) (sql.RowIter, error) {
 	switch typedPartition := partition.(type) {
 	case doltTablePartition:
@@ -493,7 +478,6 @@ var _ doltTableInterface = (*WritableDoltTable)(nil)
 
 // Internal interface for declaring the interfaces that writable dolt tables are expected to implement
 type doltTableInterface interface {
-	sql.Table2
 	sql.UpdatableTable
 	sql.DeletableTable
 	sql.InsertableTable
@@ -1331,7 +1315,7 @@ func (t *AlterableDoltTable) RewriteInserter(
 						if !sqltypes.IsText(newColumn.Type) {
 							// drop prefix lengths if column is not a string type
 							prefixLengths[i] = 0
-						} else if uint32(prefixLengths[i]) > newColumn.Type.MaxTextResponseByteLength() {
+						} else if uint32(prefixLengths[i]) > newColumn.Type.MaxTextResponseByteLength(ctx) {
 							// drop prefix length if prefixLength is too long
 							prefixLengths[i] = 0
 						}
