@@ -23,6 +23,8 @@ import (
 	"path/filepath"
 	"sync"
 
+	"github.com/dolthub/swiss"
+
 	"github.com/dolthub/dolt/go/store/hash"
 )
 
@@ -35,8 +37,11 @@ const (
 
 	chunkJournalAddr = "vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv"
 
-	journalIndexFileName        = "journal.idx"
-	journalIndexDefaultMaxNovel = 4096
+	journalIndexFileName = "journal.idx"
+
+	// journalIndexDefaultMaxNovel determines how often we flush
+	// records qto the out-of-band journal index file.
+	journalIndexDefaultMaxNovel = 16384
 )
 
 var (
@@ -152,8 +157,8 @@ type journalWriter struct {
 
 var _ io.Closer = &journalWriter{}
 
-// bootstrapJournal reads the journal file collecting a recLookup for each record and
-// returning the latest committed root hash.
+// bootstrapJournal reads in records from the journal file and the journal index file, initializing
+// the state of the journalWriter. It returns the most recent root hash for the journal.
 func (wr *journalWriter) bootstrapJournal(ctx context.Context) (last hash.Hash, err error) {
 	wr.lock.Lock()
 	defer wr.lock.Unlock()
@@ -182,6 +187,7 @@ func (wr *journalWriter) bootstrapJournal(ctx context.Context) (last hash.Hash,
 		if info, err = wr.index.Stat(); err != nil {
 			return hash.Hash{}, err
 		}
+		// process the indexed portion of the journal
 		err = processIndexRecords(ctx, wr.index, info.Size(), func(o int64, r indexRec) (err error) {
 			switch r.kind {
 			case tableIndexRecKind:
@@ -199,6 +205,7 @@ func (wr *journalWriter) bootstrapJournal(ctx context.Context) (last hash.Hash,
 				}
 				// record a high-water-mark for the indexed portion of the journal
 				wr.indexed = int64(r.end)
+				wr.ranges = wr.ranges.flatten()
 				// todo: uncompressed size
 			default:
 				return fmt.Errorf("unknown index record kind (%d)", r.kind)
@@ -211,7 +218,7 @@ func (wr *journalWriter) bootstrapJournal(ctx context.Context) (last hash.Hash,
 				return
 			}
 		}
-		wr.ranges.flatten()
+		wr.ranges = wr.ranges.flatten()
 	}
 
 	// process the non-indexed portion of the journal starting at |wr.indexed|,
@@ -328,6 +335,8 @@ func (wr *journalWriter) commitRootHash(root hash.Hash) error {
 	return err
 }
 
+// flushIndexRecord writes a new record to the out-of-band journal index file. Index records
+// accelerate journal bootstrapping by reducing the amount of the journal that must be processed.
 func (wr *journalWriter) flushIndexRecord(root hash.Hash, end int64) (err error) {
 	payload := serializeLookups(wr.ranges.novelLookups())
 	buf := make([]byte, journalIndexRecordSize(payload))
@@ -335,7 +344,7 @@ func (wr *journalWriter) flushIndexRecord(root hash.Hash, end int64) (err error)
 	if _, err = wr.index.Write(buf); err != nil {
 		return err
 	}
-	wr.ranges.flatten()
+	wr.ranges = wr.ranges.flatten()
 	// set a new high-water-mark for the indexed portion of the journal
 	wr.indexed = end
 	return
@@ -465,7 +474,7 @@ func (wr *journalWriter) Close() (err error) {
 		return err
 	}
 	if wr.index != nil {
-		wr.index.Close()
+		_ = wr.index.Close()
 	}
 	if cerr := wr.journal.Sync(); cerr != nil {
 		err = cerr
@@ -476,58 +485,67 @@ func (wr *journalWriter) Close() (err error) {
 	return
 }
 
+// A rangeIndex maps chunk addresses to read Ranges in the chunk journal file.
 type rangeIndex struct {
-	novel  map[addr]Range
-	cached map[addr]Range
+	// novel Ranges represent most recent chunks written to
+	// the journal. These Ranges have not yet been writen to
+	// a journal index record.
+	novel *swiss.Map[addr, Range]
+
+	// cached Ranges are bootstrapped from an out-of-band journal
+	// index file. To save memory, these Ranges are keyed by a 16-byte
+	// prefix of their addr which is assumed to be globally unique
+	cached *swiss.Map[addr16, Range]
+}
+
+type addr16 [16]byte
+
+func toAddr16(full addr) (prefix addr16) {
+	copy(prefix[:], full[:])
+	return
 }
 
 func newRangeIndex() rangeIndex {
 	return rangeIndex{
-		novel:  make(map[addr]Range),
-		cached: make(map[addr]Range),
+		novel:  swiss.NewMap[addr, Range](journalIndexDefaultMaxNovel),
+		cached: swiss.NewMap[addr16, Range](0),
 	}
 }
 
 func (idx rangeIndex) get(a addr) (rng Range, ok bool) {
-	rng, ok = idx.novel[a]
+	rng, ok = idx.novel.Get(a)
 	if !ok {
-		rng, ok = idx.cached[a]
+		rng, ok = idx.cached.Get(toAddr16(a))
 	}
 	return
 }
 
 func (idx rangeIndex) put(a addr, rng Range) {
-	idx.novel[a] = rng
-}
-
-func (idx rangeIndex) iter(cb func(addr, Range)) {
-	for a, r := range idx.novel {
-		cb(a, r)
-	}
-	for a, r := range idx.cached {
-		cb(a, r)
-	}
+	idx.novel.Put(a, rng)
 }
 
 func (idx rangeIndex) count() uint32 {
-	return uint32(len(idx.novel) + len(idx.cached))
+	return uint32(idx.novel.Count() + idx.cached.Count())
 }
 
 func (idx rangeIndex) novelCount() int {
-	return len(idx.novel)
+	return idx.novel.Count()
 }
 
 func (idx rangeIndex) novelLookups() (lookups []lookup) {
-	lookups = make([]lookup, 0, len(idx.novel))
-	for a, r := range idx.novel {
+	lookups = make([]lookup, 0, idx.novel.Count())
+	idx.novel.Iter(func(a addr, r Range) (stop bool) {
 		lookups = append(lookups, lookup{a: a, r: r})
-	}
+		return
+	})
 	return
 }
 
-func (idx rangeIndex) flatten() {
-	for a, r := range idx.novel {
-		idx.cached[a] = r
-		delete(idx.novel, a)
-	}
+func (idx rangeIndex) flatten() rangeIndex {
+	idx.novel.Iter(func(a addr, r Range) (stop bool) {
+		idx.cached.Put(toAddr16(a), r)
+		return
+	})
+	idx.novel = swiss.NewMap[addr, Range](journalIndexDefaultMaxNovel)
+	return idx
 }
