@@ -62,6 +62,22 @@ func (sm SerialMessage) HumanReadableString() string {
 		mapbytes := msg.AddressMapBytes()
 		fmt.Fprintf(ret, "StoreRoot{%s}", SerialMessage(mapbytes).HumanReadableString())
 		return ret.String()
+	case serial.StashListFileID:
+		msg := serial.GetRootAsStashList([]byte(sm), serial.MessagePrefixSz)
+		ret := &strings.Builder{}
+		mapbytes := msg.AddressMapBytes()
+		fmt.Fprintf(ret, "StashList{%s}", SerialMessage(mapbytes).HumanReadableString())
+		return ret.String()
+	case serial.StashFileID:
+		msg := serial.GetRootAsStash(sm, serial.MessagePrefixSz)
+		ret := &strings.Builder{}
+		fmt.Fprintf(ret, "{\n")
+		fmt.Fprintf(ret, "\tBranchName: %s\n", msg.BranchName())
+		fmt.Fprintf(ret, "\tDesc: %s\n", msg.Desc())
+		fmt.Fprintf(ret, "\tStashRootAddr: #%s\n", hash.New(msg.StashRootAddrBytes()).String())
+		fmt.Fprintf(ret, "\tHeadCommitAddr: #%s\n", hash.New(msg.HeadCommitAddrBytes()).String())
+		fmt.Fprintf(ret, "}")
+		return ret.String()
 	case serial.TagFileID:
 		return "Tag"
 	case serial.WorkingSetFileID:
@@ -163,7 +179,7 @@ func (sm SerialMessage) HumanReadableString() string {
 	}
 }
 
-func (sm SerialMessage) Less(nbf *NomsBinFormat, other LesserValuable) (bool, error) {
+func (sm SerialMessage) Less(ctx context.Context, nbf *NomsBinFormat, other LesserValuable) (bool, error) {
 	if v2, ok := other.(SerialMessage); ok {
 		return bytes.Compare(sm, v2) == -1, nil
 	}
@@ -195,6 +211,28 @@ func (sm SerialMessage) WalkAddrs(nbf *NomsBinFormat, cb func(addr hash.Hash) er
 			mapbytes := msg.AddressMapBytes()
 			return SerialMessage(mapbytes).WalkAddrs(nbf, cb)
 		}
+	case serial.StashListFileID:
+		var msg serial.StashList
+		err := serial.InitStashListRoot(&msg, []byte(sm), serial.MessagePrefixSz)
+		if err != nil {
+			return err
+		}
+		if msg.AddressMapLength() > 0 {
+			mapbytes := msg.AddressMapBytes()
+			return SerialMessage(mapbytes).WalkAddrs(nbf, cb)
+		}
+	case serial.StashFileID:
+		var msg serial.Stash
+		err := serial.InitStashRoot(&msg, []byte(sm), serial.MessagePrefixSz)
+		if err != nil {
+			return err
+		}
+		if err = cb(hash.New(msg.StashRootAddrBytes())); err != nil {
+			return err
+		}
+		if err = cb(hash.New(msg.HeadCommitAddrBytes())); err != nil {
+			return err
+		}
 	case serial.TagFileID:
 		var msg serial.Tag
 		err := serial.InitTagRoot(&msg, []byte(sm), serial.MessagePrefixSz)
@@ -301,19 +339,7 @@ func (sm SerialMessage) WalkAddrs(nbf *NomsBinFormat, cb func(addr hash.Hash) er
 		}
 
 		mapbytes := msg.PrimaryIndexBytes()
-
-		if nbf == Format_DOLT_DEV {
-			dec := newValueDecoder(mapbytes, nil)
-			v, err := dec.readValue(nbf)
-			if err != nil {
-				return err
-			}
-			return v.walkRefs(nbf, func(ref Ref) error {
-				return cb(ref.TargetHash())
-			})
-		} else {
-			return SerialMessage(mapbytes).WalkAddrs(nbf, cb)
-		}
+		return SerialMessage(mapbytes).WalkAddrs(nbf, cb)
 	case serial.CommitFileID:
 		parents, err := SerialCommitParentAddrs(nbf, sm)
 		if err != nil {
